package liste;

import java.util.Random;

/**
 * Classe con alcune funzioni di utilita' su liste
 * le cui etichette contengono interi
 * 
 * @author Stefano Guerrini
 * @version Marzo 2009
 */
public final class ListeIntLib
{    
    /**
     * crea un nuovo nodo con una etichetta intera e campo next pari a null
     * 
     * @param n         valore dell'etcihetta
     * @return          il nuovo nodo
     */
    public static Nodo nuovoNodoInt(int n)
    {
        return nuovoNodoInt(n, null);
    }

    /**
     * crea un nuovo nodo con una etichetta intera e lo aggiunge in
     * testa a una lista esistente
     * 
     * @param n         valore dell'etcihetta
     * @param nd        lista a cui aggiungere il nodo
     * @return          il nuovo nodo
     */
    public static Nodo nuovoNodoInt(int n, Nodo nd)
    {
        return new Nodo(new EtichettaNodo(n), nd);
    }

    /**
     * genera una lista con gli interi di un intervallo
     * 
     * @param first     valore iniziale intervallo
     * @param last      valore finale intervallo
     * @return          una lista con gli interi compresi tra first e last inclusi
     */
    public static Nodo intervalloLista(int first, int last)
    {
        Nodo lista = null;
        
        while (last >= first)
            lista = nuovoNodoInt((last--), lista);

        return lista;
    }

    private static Random generator = new Random();
    
    /**
     * genera una lista di interi di lunghezza data contenente elementi random
     * 
     * @param lungh     lunghezza della lista da generare
     * @return          la nuova lista
     */
    public static Nodo randomLista(int lungh)
    {
        Nodo lista = null;
        
        while (lungh-- > 0)
            lista = nuovoNodoInt(generator.nextInt(), lista);

        return lista;
    }
    
    /**
     * genera una lista di interi di lunghezza data contenete elementi random
     * appartenenti a un intervallo
     * 
     * @param lungh     lunghezza della lista da generare
     * @param min       minimo dell'intervallo
     * @paaram max      massimo del'intervallo
     * @return          la nuova lista
     */
    public static Nodo randomLista(int lungh, int min, int max)
    {
        if (min > max)
            return null; // intervallo vuoto
            
        Nodo lista = null;
        
        while (lungh-- > 0)
            lista = nuovoNodoInt(generator.nextInt(max-min+1)+min, lista);

        return lista;
    }
    
    /**
     * inserisce un elemento nella posizione della lista tale che l'elemento
     * della lista nella posizione e' maggiore di quello da inserire, mentre
     * tutti gli elementi prima di tale posizione sono minori.
     * -- versione ricorsiva --
     * 
     * @param lista     lista in cui inserire l'elemento
     * @param n         valore intero da inserire
     * @return          la lista risultato
     */
    public static Nodo insInOrdineRic(Nodo lista, int n)
    {
        if (lista == null || lista.label().getIntVal() >= n)
            lista = nuovoNodoInt(n, lista);
        else 
            lista.setNext(insInOrdine(lista.next(), n));
            
        return lista;            
    }
    
    /**
     * inserisce un elemento nella posizione della lista tale che l'elemento
     * della lista nella posizione e' maggiore di quello da inserire, mentre
     * tutti gli elementi prima di tale posizione sono minori.
     * -- versione iterativa --
     * 
     * @param lista     lista in cui inserire l'elemento
     * @param n         valore intero da inserire
     * @return          la lista risultato
     */
    public static Nodo insInOrdine(Nodo lista, int n)
    {        
        // lista vuota: risultato lista composta solo dal nuovo nodo
        if (lista == null || lista.label().getIntVal() >= n)
            return nuovoNodoInt(n, lista);
        
        // lista nuovo:  ricerca la posizione in cui inserire il nodo
        // curr: nodo corrente con cui confrontare il valore da inserire
        // pred: nodo che precede curr
        Nodo pred = lista, curr = pred.next();
        while (curr != null && curr.label().getIntVal() < n)
        {
            pred = curr;
            curr = pred.next();
        }

        // il nuovo nodo va inserito tra pred e curr
        pred.setNext(nuovoNodoInt(n, curr));
        
        // il risultato e' sempre la vecchia lista
        return lista;            
    }
    
    /**
     * insertion sort
     * usa la insInOrdine per copiare una lista ordinandola
     * in ordine crescente
     * 
     * @param lista     lista da ordinare
     * @return          nuova lista ordinata
     */
    public static Nodo insSort(Nodo lista)
    {
        Nodo ordinata = null;

        while (lista != null)
        {
            ordinata = insInOrdine(ordinata, lista.label().getIntVal());
            lista = lista.next();
        }
        return ordinata;            
    }
    
    /**
     * insertion sort
     * usa la insInOrdineRic per copiare una lista ordinandola
     * in ordine crescente
     * -- NB. Usa la versione ricorsiav di insInOrdine --
     * 
     * @param lista     lista da ordinare
     * @return          nuova lista ordinata
     */
    public static Nodo insSortRic(Nodo lista)
    {
        Nodo ordinata = null;

        while (lista != null)
        {
            ordinata = insInOrdineRic(ordinata, lista.label().getIntVal());
            lista = lista.next();
        }
        return ordinata;            
    }
}
