package liste.itera;

import liste.Nodo;
import liste.EtichettaNodo;

/**
 * Classe con alcune funzioni su liste definite usando l'iteratore
 * 
 * @author Stefano Guerrini
 * @version Marzo 2009
 */
public final class IteraLib
{
    /**
     * copia una lista usando l'iteratore
     * 
     * @param lista     lista da copiare
     * @return          la nuova lista
     */
    public static Nodo copiaLista(Nodo lista)
    {
        // lista vuota, non fare null
        if (lista == null)
            return null;

        Nodo nuova = new Nodo(lista.label().clona());
        Nodo last = nuova; // ultimo nodo della nuova lista
        IteraSuLista iter = new IteraSuLista(lista.next());
        
        while (iter.hasNext())
        {
            // leggo il nodo e avanzo l'iteratore
            Nodo nd = iter.next();
            // duplico l'oggetto dell'etichetta e lo aggiungo
            // in coda alla nuova lista
            last.setNext(new Nodo(nd.label().clona()));
            last = last.next(); // avanza ref all'ultimo nodo
        }
        
        return nuova;
    }

    /**
     * rovescia una lista creandone una nuova copia
     * usando l'iteratore
     * 
     * @param lista     lista da copiare
     * @return          la nuova lista
     */
    public static Nodo revLista(Nodo lista)
    {
        Nodo nuova = null;
        IteraSuLista iter = new IteraSuLista(lista);
        
        while (iter.hasNext())
        {
            // leggo il nodo e avanzo l'iteratore
            Nodo nd = iter.next();
            // duplichiamo anche l'oggetto dell'etichetta
            nuova = new Nodo(nd.label().clona(), nuova);
        }
        
        return nuova;
    }

    /**
     * stampa una lista usando l'iteratore
     * 
     * @param lista     lista da stampare
     */
    public static void stampaLista(Nodo lista)
    {
        IteraSuLista iter = new IteraSuLista(lista);
        
        while (iter.hasNext())
        {
            // legge il next, stampa l'etichetta e avanza l'iteratore
            System.out.print(iter.next().label().toString() + " ");
        }
    }
}
